#include "botdll.h"

#include "datatypes.h"
#include "algorithms.h"
#include "host.h"
#include "clientprot.h"

#define WIN32_LEAN_AND_MEAN
#include <windows.h>


void DLLImports::clearImports()
{
	talk(makeTerm());

	DLL_TALK = NULL;
}

void __stdcall listen(BotEvent &event)
{
	Host *h = (Host*)event.handle;

	switch (event.code)
	{
	case EVENT_Echo:
		{
			char *msg = (char*)event.p[0];

			h->logEvent("Ext: %s", msg);
		}
		break;
	case EVENT_Say:
		{
			int mode = *(int*)&event.p[0];
			int sound = *(int*)&event.p[1];
			int ident = *(int*)&event.p[2];
			char *msg = (char*)event.p[3];

			h->tryChat(mode, sound, ident, msg);
		}
		break;
	case EVENT_Ship:
		{
			int ship = *(int*)&event.p[0];

			h->postRR(generateChangeShip((Ship_Types)ship));
		}
		break;
	case EVENT_GrabFlag:
		{
			int flag = *(int*)&event.p[0];

			h->postRR(generateFlagRequest(flag));
		}
		break;
	case EVENT_SendPosition:
		{
			h->sendPosition();
		}
		break;
	case EVENT_DropFlags:
		{
			h->postRR(generateFlagDrop());
		}
		break;
	default:
		{
			String s;
			s += "Unknown message from DLL (";
			s += event.code;
			s += ")  Version mismatch?";
			h->logEvent("%s", s);
		}
		break;
	}
}

bool DLLImports::importLibrary(char *file)
{
	String s = file;

	// Avoid directory traversal exploits.
	trimString('/', s);
	trimString('\\', s);

	// Make sure that a .DLL extension is used.
	tolower(s.msg);
	if (s.right(4) != ".dll")
		s += ".dll";

	HINSTANCE DLLhInst = LoadLibrary(file);

	if (DLLhInst)
	{
		DLL_TALK = (CALL_TALK)GetProcAddress(DLLhInst, (LPCSTR)1);

		talk(makeInit(&listen, &h->playerlist, &h->flagList));
		if (h->inArena)
			talk(makeArenaEnter(h->botInfo.initialArena, h->Me));

		return true;
	}

	return false;
}

void DLLImports::talk(BotEvent &event)
{
	if (DLL_TALK)
	{
		event.handle = h;

		DLL_TALK(event);
	}
}

DLLImports::DLLImports(Host &host)
{
	DLL_TALK = NULL;

	h = &host;
}

DLLImports::DLLImports(DLLImports &imports)
{
	DLL_TALK = imports.DLL_TALK;
}

DLLImports::~DLLImports()
{
	clearImports();
}
